#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/2/22 9:58
# @Author  : chenjw
# @Site    : 
# @File    : mongo.py
# @Software: PyCharm Community Edition
# @Desc    :  pymongo already has it's poolSize

import pymongo
import decimal
import datetime


class Mongo:
    def __init__(self, host, port, dbName, poolSize=8):
        self.host = host
        self.port = port
        self.dbName = dbName
        self.poolSize = poolSize
        self.client = None
        self.db = None

    def connect(self):
        self.client = pymongo.MongoClient(self.host, self.port, connect=False, maxPoolSize=self.poolSize)
        self.db = self.client[self.dbName]

    def collection(self, c_name):
        if self.db is None:
            self.connect()
        return self.db[c_name]

    def close(self):
        if self.client is not None:
            try:
                self.client.close()
            except:
                pass

    def toMap(self, keys, values):
        if len(keys) != len(values):
            raise Exception('len of keys and values is not the same')
        keys = self.toLower(keys)
        ret_map = {}
        for index in range(len(keys)):
            key = keys[index]
            value = values[index]
            if isinstance(value, decimal.Decimal):
                value = float(value)
            if isinstance(value, datetime.datetime):
                value = int(value.timestamp() * 1000)
            ret_map[key] = value
        return ret_map


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/2/21 14:20
# @Author  : chenjw
# @Site    : 
# @File    : util.py
# @Software: PyCharm Community Edition
# @Desc    :  do what

import time
import decimal

# 获取当天时间
def zeroStr(added=0):
    '''
    :param added: offset,最好是 86400 的整倍数
    :return: 返回时间 like ‘2018-01-01’
    '''
    return time.strftime('%Y-%m-%d', time.localtime(time.time() + added))

def round(f,r):
    decimal.getcontext().rounding = decimal.ROUND_HALF_UP
    b = decimal.Decimal('%s'%(f), decimal.getcontext())
    return float(b.__round__(r))

if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/2/22 10:14
# @Author  : chenjw
# @Site    : 
# @File    : hive.py
# @Software: PyCharm Community Edition
# @Desc    :  without poolsize

from impala.dbapi import connect as hive_connect


class Hive:
    def __init__(self, host, port, user, password, database, auth_mechanism):
        self.host = host
        self.port = port
        self.database = database
        self.auth_mechanism = auth_mechanism
        self.user = user
        self.password = password
        self.conn = None

    def connect(self):
        self.conn = hive_connect(host=self.host,
                                 port=self.port,
                                 user=self.user,
                                 password=self.password,
                                 kerberos_service_name='test',
                                 database=self.database,
                                 auth_mechanism=self.auth_mechanism)

    def getConn(self):
        if self.conn is None:
            self.connect()
        return self.conn

    def ExecQuery(self, hql):
        with self.getConn().cursor() as cursor:
            cursor.execute(hql)
            result = cursor.fetchall()
            return result

    def close(self):
        if self.conn is not None:
            try:
                self.conn.close()
            except:
                pass


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/4/23 10:06
# @Author  : chenjw
# @Site    : 
# @File    : smtp.py
# @Software: PyCharm Community Edition
# @Desc    :  send mail by smtp

import logging

from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.header import Header
import smtplib
import os


class SIMPLESMTP:
    def __init__(self, fromAddress, password, smtpServer, port):
        '''
        :param fromAddress: username of email eg:123@qq.com
        :param password: password of username
        :param smtpServer:  host of mail eg:smtp.qq.com
        :param port: port of mail send
        '''
        self.fromAddress = fromAddress
        self.password = password
        self.smtpServer = smtpServer
        self.port = port


    def combineContent(self, receivers, subject, context, context_type='plain', encode='utf-8', charset='base64',
                       attachs=[]):
        '''
        :param receivers: who is going to receive the mail , AryStr
        :param subject: title of mail
        :param context: context of mail
        :param context_type: context type of mail,common to be plain,html and so on
        :param encode: common to be utf-8
        :param charset: attachs charset
        :param attachs: files arystr
        :return:
        '''
        if attachs is None or (isinstance(attachs, list) and len(attachs) == 0):
            message = MIMEText(context, context_type, encode)
            message['From'] = Header(self.fromAddress, encode)
            message['To'] = Header(",".join(receivers), encode)
            message['Subject'] = Header(subject, encode)
        else:
            message = MIMEMultipart()
            message['From'] = Header(self.fromAddress, encode)
            message['To'] = Header(",".join(receivers), encode)
            message['Subject'] = Header(subject, encode)
            message.attach(MIMEText(context, context_type, encode))
            for fileName in attachs:
                attach = MIMEText(open(fileName, 'rb').read(), charset, encode)
                attach["Content-Type"] = 'application/octet-stream'
                attach["Content-Disposition"] = '''attachment; filename="%s"''' % fileName.split(os.pathsep)[-1]
                message.attach(attach)
        return message

    def send(self, message, receivers):
        '''
        :param message:
        :param receivers:
        :return:
        '''
        try:
            smtp = smtplib.SMTP()
            smtp.connect(self.smtpServer, self.port)
            smtp.login(self.fromAddress, self.password)
            smtp.sendmail(self.fromAddress, receivers, message.as_string())
        except smtplib.SMTPException as e:
            logging.error("call [smtp.sendmail] error:%s", e)

    def sendEmail(self, receivers, subject, context, context_type='plain', encode='utf-8', charset='base64',
                  attachs=[]):
        try:
            message = self.combineContent(receivers, subject, context, context_type, encode, charset, attachs)
            smtp = smtplib.SMTP()
            smtp.connect(self.smtpServer, self.port)
            smtp.login(self.fromAddress, self.password)
            smtp.sendmail(self.fromAddress, receivers, message.as_string())
        except smtplib.SMTPException as e:
            logging.error("call [smtp.sendmail] error:%s", e)


if __name__ == '__main__':
    username = 'username'
    pwd = 'pwd'
    mailServer = 'smtp.exmail.qq.com'
    port = 25
    receivers = ['781354598@qq.com']

    simple = SIMPLESMTP(username, pwd, mailServer, port)
    simple.sendEmail(receivers, '文本标题', '内容')
    simple.sendEmail(receivers, 'html标题', '''
    hello ff
    <html><body><h1>hello</h1></body></html>
    ''', context_type='html')
    simple.sendEmail(receivers, '附件', '附件哦哦', attachs=['__init__.py'])

    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/3/1 10:05
# @Author  : chenjw
# @Site    : 
# @File    : log.py
# @Software: PyCharm Community Edition
# @Desc    :  do what

import logging


def init(logname='log.log'):
    logging.basicConfig(level=logging.INFO,
                        format='%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s %(message)s',
                        datefmt='%a, %d %b %Y %H:%M:%S',
                        filename=logname,
                        filemode='w')


if __name__ == '__main__':
    init()
    logging.info('aa')
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/1/30 17:30
# @Author  : chenjw
# @Site    : 
# @File    : monitor.py
# @Software: PyCharm Community Edition
# @Desc    :  do what
import threading
import queue
import time
from ccommon import jsonParse


class ItemCost:
    def __init__(self, min=-1, max=0, count=0, cost=0, ave=0, ms=0):
        self.min = min
        self.max = max
        self.count = count
        self.cost = cost
        self.ave = ave
        self.range = {}
        self.ms = ms

    def addItem(self, num):
        if isinstance(num, int) or isinstance(num, float) and num >= 0:
            if self.min < 0 and num >= 0:
                self.min = num
            elif num < self.min:
                self.min = num
            if num > self.max:
                self.max = num
            self.cost += num
            self.count += 1
            if self.ms > 0:
                range_index = str(int(num / self.ms))
                if range_index not in self.range.keys():
                    self.range[range_index] = 0
                self.range[range_index] += 1

    def loadSum(self):
        if self.count == 0:
            return {'min': 0, 'max': 0, 'count': 0, 'cost': 0, 'ave': 0, 'range': {}}
        else:
            return {'min': self.min, 'max': self.max, 'count': self.count, 'cost': self.cost,
                    'ave': float(self.cost) / float(self.count), 'range': self.range}


class Item():
    def __init__(self, key=None, btime=None, etime=None, cost=None):
        self.key = key
        self.btime = btime
        self.etime = etime
        self.cost = cost


class Monitor():
    def __init__(self, ms=0):
        self.ms = ms
        self._queue = queue.Queue()
        self.lock = threading.Lock()
        self.cost = {}
        self.t = threading.Thread(target=self.syncRecord)
        self.t.start()

    def addItem(self, key, cost):
        self._queue.put(Item(key=key, cost=cost))

    def addItem2(self, key, btime, etime):
        self._queue.put(Item(key=key, btime=btime, etime=etime))

    def syncRecord(self):
        while True:
            if self._queue.empty() is True:
                time.sleep(0.00000001)
            else:
                item = self._queue.get()
                self.recordItem(item)

    def recordItem(self, item):
        self.lock.acquire(1)
        try:
            if item.key is None or (isinstance(item.key, str) and item.key == ''):
                raise
            if item.key not in self.cost.keys():
                self.cost[item.key] = ItemCost(ms=self.ms)
            if item.cost is not None and (isinstance(item.cost, int), isinstance(item.cost, float)):
                self.cost[item.key].addItem(item.cost)
                raise
            if item.btime is not None and (
                    isinstance(item.btime, int), isinstance(item.btime, float)) and item.etime is not None and (
                    isinstance(item.etime, int), isinstance(item.etime, float)) and item.etime >= item.btime:
                self.cost[item.key].addItem(item.etime - item.btime)
                raise
        except:
            pass
        finally:
            self.lock.release()

    def loadRecord(self):
        self.lock.acquire(1)
        ret = {}
        try:
            for k, itemCost in self.cost.items():
                ret[k] = itemCost.loadSum()
        finally:
            self.lock.release()
            return jsonParse.JsonParse(ret)


if __name__ == '__main__':
    m = Monitor(1000)
    for i in range(10000):
        m.addItem('key1', 123)
    time.sleep(1)
    print(m.loadRecord().toString())

    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/5/10 9:35
# @Author  : chenjw
# @Site    : 
# @File    : tapd.py
# @Software: PyCharm Community Edition
# @Desc    :  do what

from urllib.parse import urlencode
from requests import auth
import time
import threading
from tapdApi import tapd_param
from ccommon.httpSimple import HttpSimple


class Tapd:
    def __init__(self, usr, pwd, delay=1000):
        self.usr = usr
        self.pwd = pwd
        self.delay = delay
        self.lastRequestTime = 0
        self.lock = threading.Lock()

    def retAuth(self):
        return auth.HTTPBasicAuth(self.usr, self.pwd)

    def checkApiStatus(self, _json):
        if _json.Int('status') != 1:
            raise Exception('[Tapd] checkApiStatus eager %s but indeed %s' % (1, _json.Int('status')))

    # 时间戳 毫秒
    def now(self):
        return int(round(time.time() * 1000))

    # 等待
    def wait(self):
        self.lock.acquire()
        try:
            if self.lastRequestTime == 0:
                self.lastRequestTime = self.now()
            else:
                sub_time = self.now() - self.lastRequestTime
                if sub_time < self.delay:
                    time.sleep(float(self.delay - sub_time) / float(1000))
                self.lastRequestTime = self.now()
        except:
            pass
        finally:
            self.lock.release()

    # 获取所有的项目
    def loadAllWorkspace(self, company_id):
        '''
        :param company_id:    N   int     公司 id
        :return:    list
            {
                "Workspace": {
                    "id": "20003271",
                    "name": "the_preoject_name",
                    "pretty_name": "20003271",
                    "status": "normal",
                    "secrecy": "0",
                    "created": "2015-05-08 16:20:01",
                    "creator_id": "2000005851",
                    "member_count": 14,
                    "creator": "username (mail@host.name)"
                }
            }
        '''
        url = 'https://api.tapd.cn/workspaces/projects?'
        param = tapd_param.TapdParam()
        param.setValue(company_id, int, 'company_id', True, {param.Int: {param.gt: 0}})
        self.wait()
        url = '%s%s' % (url, urlencode(param.data))
        _json = HttpSimple(url).addMethod(HttpSimple.method_get).addAuth(
            self.retAuth()).run().retJson()
        self.checkApiStatus(_json)
        return [ain_json.Dict for ain_json in _json.AryJson('data')]

    def loadWorkspaceUsers(self, workspace_id, fields=None):
        '''
        :param workspace_id:    N   int     项目 id
        :param fields:          O   str     需要查的字段值     user,role_id,email
        :return:    list
            {
                "UserWorkspace": {
                    "user": "wiki",
                    "role_id": [
                        "1000000000000000002"
                    ]
                }
            }
        '''
        url = 'https://api.tapd.cn/workspaces/users?'
        param = tapd_param.TapdParam()
        param.setValue(workspace_id, int, 'workspace_id', True, {param.Int: {param.gt: 0}})
        param.setValue(fields, str, 'fields')
        self.wait()
        url = '%s%s' % (url, urlencode(param.data))
        _json = HttpSimple(url).addMethod(HttpSimple.method_get).addAuth(
            self.retAuth()).run().retJson()
        self.checkApiStatus(_json)
        return [ain_json.Dict for ain_json in _json.AryJson('data')]

    def loadRelation(self, workspace_id, source_type=None, target_type=None, source_id=None, target_id=None):
        url = 'https://api.tapd.cn/relations?'
        param = tapd_param.TapdParam()
        param.setValue(workspace_id, int, 'workspace_id', True, {param.Int: {param.gt: 0}})
        param.setValue(source_type, str, 'source_type')
        param.setValue(target_type, str, 'target_type')
        param.setValue(source_id, int, 'source_id', False, {param.Int: {param.gt: 0}})
        param.setValue(target_id, int, 'target_id', False, {param.Int: {param.gt: 0}})
        self.wait()
        url = '%s%s' % (url, urlencode(param.data))
        _json = HttpSimple(url).addMethod(HttpSimple.method_get).addAuth(
            self.retAuth()).run().retJson()
        self.checkApiStatus(_json)
        return [ain_json.Dict for ain_json in _json.AryJson('data')]

    def statusMapBasic(self, workspace_id, system='story'):
        '''
        :param workspace_id:        N       int         项目 ID
        :return:
        '''
        param = tapd_param.TapdParam()
        param.setValue(workspace_id, int, 'workspace_id', True, {param.Int: {param.gt: 0}})
        param.setValue(system, str, 'system')
        url = 'https://api.tapd.cn/workflows/status_map?'
        self.wait()
        url = '%s%s' % (url, urlencode(param.data))

        _json = HttpSimple(url).addMethod(HttpSimple.method_get).addAuth(self.retAuth()).run().retJson()

        self.checkApiStatus(_json)
        return _json.Json('data').Dict

    def dateCombine(self, key, choice_map):
        ret = ''
        if isinstance(choice_map, dict):
            le = None
            ge = None
            if 'le' in choice_map.keys():
                le = choice_map['le']
            if 'ge' in choice_map.keys():
                ge = choice_map['ge']
            if le is not None:
                ret = '&%s=<%s' % (key, le)
            if ge is not None:
                ret = '&%s=>%s' % (key, ge)
        return ret

    def dateTimeCombine(self, key, choice_map):
        ret = ''
        if isinstance(choice_map, dict):
            btime = None
            etime = None
            if 'btime' in choice_map.keys():
                btime = choice_map['btime']
            if 'etime' in choice_map.keys():
                etime = choice_map['etime']
            if btime is not None and etime is not None:
                # ret = '&%s=%s~%s' % (key, btime, etime)
                ret = '&' + urlencode({key: '%s~%s' % (btime, etime)})
            elif btime is not None:
                # ret = '&%s=>%s' % (key, btime)
                ret = '&' + urlencode({key: '>%s' % (btime)})
            elif etime is not None:
                # ret = '&%s=<%s' % (key, etime)
                ret = '&' + urlencode({key: '<%s' % (etime)})
        return ret


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/2/22 10:03
# @Author  : chenjw
# @Site    : 
# @File    : redisdb.py
# @Software: PyCharm Community Edition
# @Desc    :  do what

import redis


class Redis:
    def __init__(self, host, port, db=0, password=None):
        self.host = host
        self.port = port
        self.db = db
        self.password = password
        self.pool = None

    def connection(self):
        self.pool = redis.ConnectionPool(host=self.host, port=self.port, db=self.db, password=self.password)

    def getRedis(self):
        if self.pool is None:
            self.connection()
        return redis.Redis(connection_pool=self.pool)

    def push(self, key, infos):
        r = self.getRedis()
        for info in infos:
            r.lpush(key, info)

    def pop(self, key):
        return self.getRedis().rpop(key)


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/3/8 18:17
# @Author  : chenjw
# @Site    : 
# @File    : httpSimple.py
# @Software: PyCharm Community Edition
# @Desc    :  simple http requests,support chain

import base64
import requests
import time
from requests.auth import HTTPBasicAuth
from bs4 import BeautifulSoup
from ccommon import jsonParse


class HttpSimple:
    method_get = 'get'
    method_post = 'post'



    def __init__(self, url):
        self.url = url
        self.status_code = 200
        self.method = HttpSimple.method_get
        self.data = None
        self.headers = None
        self.basicAuth = None
        self.rsp = None
        self.timeout=5
        self.requestCost = 0

    def setTimeout(self,timeout):
        self.timeout = timeout
        return self

    def addData(self, data):
        self.data = data
        return self

    def addDataSingle(self, key, value):
        if self.data is None:
            self.data = {}
        if isinstance(key, str) is True:
            self.headers[key] = value
        return self

    def addHeaders(self, headers):
        if isinstance(headers, dict) is True:
            self.headers = headers
        return self

    def addHeadersSingle(self, key, value):
        if self.headers is None:
            self.headers = {}
        if isinstance(key, str) is True:
            self.data[key] = value
        return self

    def addAuth(self, auth=None, user=None, pwd=None):
        if auth is not None:
            self.basicAuth = auth
        if isinstance(user, str) is True and isinstance(pwd, str) is True:
            pwd = str(base64.b64encode(bytes('%s:%s' % (user, pwd), 'utf-8')), 'utf-8')
            self.basicAuth = HTTPBasicAuth(user, pwd)
        return self

    def addStatusCode(self, status_code):
        if isinstance(status_code, int) is True:
            self.status_code = status_code
        return self

    def addMethod(self, method):
        if isinstance(method, str) is True:
            if method.lower() in [HttpSimple.method_get, HttpSimple.method_post]:
                self.method = method.lower()
        return self

    def run(self):
        btime = int(round(time.time() * 1000))
        if self.method == HttpSimple.method_get:
            self.rsp = requests.get(url=self.url, data=self.data, headers=self.headers, auth=self.basicAuth,timeout=self.timeout)
        else:
            self.rsp = requests.post(url=self.url, data=self.data, headers=self.headers, auth=self.basicAuth,timeout=self.timeout)
        self.requestCost = int(round(time.time() * 1000)) - btime
        self.checkStatus()
        return self

    def retCost(self):
        return self.requestCost

    def checkStatus(self):
        self._checkRsp()
        if self.rsp.status_code != self.status_code:
            raise Exception('[HttpSimple] call url(%s) by method(%s) but status_code is %s' % (
                self.url, self.method, self.rsp.status_code))

    def _checkRsp(self):
        if self.rsp is None:
            raise Exception('[HttpSimple] please call function run() at first')

    def retRspHeaders(self):
        self._checkRsp()
        return self.rsp.headers

    def retReqHeaders(self):
        self._checkRsp()
        return self.rsp.request.headers

    def retJson(self):
        self._checkRsp()
        return jsonParse.JsonParse(self.rsp.json())

    def retText(self):
        self._checkRsp()
        return self.rsp.text

    def retSoup(self):
        self._checkRsp()
        return BeautifulSoup(self.rsp.text, 'html.parser')


if __name__ == '__main__':
    # demo
    HttpSimple('url'). \
        addData({'a': 'a', 'b': 'b'}). \
        addDataSingle('c', 'c'). \
        addHeaders({'cookie': 'aaa'}). \
        addHeadersSingle('host', 'aaa.com'). \
        addMethod(HttpSimple.method_get). \
        run(). \
        retText()
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2019/1/15 9:54
# @Author  : chenjw
# @Site    : 
# @File    : tapd_task.py
# @Software: PyCharm Community Edition
# @Desc    :  do what

from urllib.parse import urlencode
from ccommon.httpSimple import HttpSimple
from tapdApi import tapd, tapd_param
import datetime


class TapdTasks(tapd.Tapd):
    def __init__(self, usr, pwd, delay=1000):
        tapd.Tapd.__init__(self, usr, pwd, delay)

    def listTasks(self, workspace_id, id=None, name=None, priority=None, status=None, iteration_id=None, owner=None,
                  cc=None, creator=None,
                  begin=None, due=None, created=None, modified=None, completed=None, effort=None, effort_completed=None,
                  remain=None, exceed=None, progress=None, story_id=None, description=None, limit=None, page=None,
                  order=None, fields=None):
        """
        :param workspace_id:        N       int         项目ID
        :param id:                  O       int         支持多ID查询、模糊匹配
        :param name:                O       str         标题	支持模糊匹配
        :param priority:            O       str         优先级	支持枚举查询
        :param status:              O       str         状态	支持枚举查询
        :param iteration_id:        O       int         迭代
        :param owner:               O       str         当前处理人	支持模糊匹配
        :param cc:                  O       str         抄送人
        :param creator:             O       str         创建人	支持模糊匹配
        :param begin:               O       date        预计开始	支持时间查询
        :param due:                 O       date        预计结束	支持时间查询
        :param created:             O       datetime    创建时间	支持时间查询
        :param modified:            O       datetime    最后修改时间
        :param completed:           O       datetime    完成时间	支持时间查询
        :param effort:              O       str         预估工时
        :param effort_completed:    O       str         完成工时
        :param remain:              O       float       剩余工时
        :param exceed:              O       float       超出工时
        :param progress:            O       int         进度
        :param story_id:            O       int         需求	支持多ID查询
        :param description:         O       str         详细描述
        :param limit:               O       int         设置返回数量限制，默认为30
        :param page:                O       int         返回当前数量限制下第N页的数据，默认为1（第一页）
        :param order:               O       str         排序规则，规则：字段名 ASC或者DESC，然后 urlencode	如按创建时间逆序：order=created%20desc
        :param fields:              O       str         置获取的字段，多个字段间以','逗号隔开
        :return:
        """
        url = 'https://api.tapd.cn/tasks?'
        param = tapd_param.TapdParam()
        param.setValue(workspace_id, int, 'workspace_id', True, {param.Int: {param.gt: 0}})
        param.setValue(id, int, 'id')
        param.setValue(name, str, 'name')
        param.setValue(priority, str, 'priority')
        param.setValue(status, str, 'status')
        param.setValue(iteration_id, int, 'iteration_id')
        param.setValue(owner, str, 'owner')
        param.setValue(cc, str, 'cc')
        param.setValue(creator, str, 'creator')
        param.setValue(begin, datetime.date, 'begin')
        param.setValue(due, datetime.date, 'due')
        param.setValue(effort, str, 'effort')
        param.setValue(effort_completed, str, 'effort_completed')
        param.setValue(remain, float, 'remain')
        param.setValue(exceed, float, 'exceed')
        param.setValue(progress, int, 'progress')
        param.setValue(story_id, int, 'story_id')
        param.setValue(description, str, 'description')
        param.setValue(limit, int, 'limit')
        param.setValue(page, int, 'page')
        param.setValue(order, str, 'order')
        param.setValue(fields, str, 'fields')

        self.wait()
        url = '%s%s' % (url, urlencode(param.data))
        url += self.dateTimeCombine('created', created)
        url += self.dateTimeCombine('modified', modified)
        url += self.dateTimeCombine('completed', completed)
        _json = HttpSimple(url).addMethod(HttpSimple.method_get).addAuth(self.retAuth()).run().retJson()
        self.checkApiStatus(_json)
        if _json.ExistP('data/Task/id'):
            return _json.JsonP('data/Task').Dict
        elif _json.ExistP('data/0/Task/id'):
            return [ain_json.Dict for ain_json in _json.AryJson('data')]
        else:
            return None


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/11/8 9:42
# @Author  : chenjw
# @Site    : 
# @File    : tapd_test_plan.py
# @Software: PyCharm Community Edition
# @Desc    :  do what

from urllib.parse import urlencode
from ccommon.httpSimple import HttpSimple
from tapdApi import tapd, tapd_param
import datetime


class TapdTestPlan(tapd.Tapd):
    def __init__(self, usr, pwd, delay=1000):
        tapd.Tapd.__init__(self, usr, pwd, delay)

    def listTestPlan(self, workspace_id, id=None, name=None, description=None,
                     version=None, owner=None, status=None, type=None, start_date=None,
                     end_date=None, creator=None, created=None, modified=None, modifier=None, limit=30, page=1,
                     order='created desc', fields=None):
        """
        :param workspace_id:        N       int         项目 ID
        :param id:                  O       int         id
        :param name:                O       str         用例名称
        :param description:         O       str         需求分类描述
        :param version:             O       str         版本
        :param owner:               O       str         测试负责人
        :param status:              O       str         用例状态 enum('close','open')
        :param type:                O       str         测试类型
        :param start_date:          O       date        开始时间
        :param end_date:            O       date        结束时间
        :param creator:             O       str         创建人
        :param created:             O       datetime    创建时间
        :param modified:            O       datetime    最后修改时间
        :param modifier:            O       string      最后修改人
        :param limit:               O       int         设置返回数量限制，默认为30
        :param page:                O       int         返回当前数量限制下第N页的数据，默认为1
        :param order:               O       str         排序规则，规则：字段名 ASC或者DESC，然后 urlencode
        :param fields:              O       str         设置获取的字段，多个字段间以','逗号隔开
        :return:
        """
        url = 'https://api.tapd.cn/test_plans?'
        param = tapd_param.TapdParam()
        param.setValue(workspace_id, int, 'workspace_id', True, {param.Int: {param.gt: 0}})
        param.setValue(id, int, 'id')
        param.setValue(name, str, 'name')
        param.setValue(description, str, 'description')
        param.setValue(version, str, 'version')
        param.setValue(owner, str, 'owner')
        param.setValue(status, str, 'status')
        param.setValue(type, str, 'type')
        param.setValue(creator, str, 'creator')
        param.setValue(modifier, str, 'modifier')
        param.setValue(limit, int, 'limit')
        param.setValue(page, int, 'page')
        param.setValue(order, str, 'order')
        param.setValue(fields, str, 'fields')
        self.wait()
        url = '%s%s' % (url, urlencode(param.data))
        url += self.dateCombine('start_date', start_date)
        url += self.dateCombine('end_date', end_date)
        url += self.dateTimeCombine('created', created)
        url += self.dateTimeCombine('modified', modified)
        # print(url)
        _json = HttpSimple(url).addMethod(HttpSimple.method_get).addAuth(self.retAuth()).run().retJson()
        self.checkApiStatus(_json)
        if _json.ExistP('data/TestPlan/id'):
            return _json.JsonP('data/TestPlan').Dict
        elif _json.ExistP('data/0/TestPlan/id'):
            return [ain_json.Dict for ain_json in _json.AryJson('data')]
        else:
            return None

    def countTestPlan(self, workspace_id, id=None, name=None, description=None,
                      version=None, owner=None, status=None, type=None, start_date=None,
                      end_date=None, creator=None, created=None, modified=None, modifier=None):
        """
        :param workspace_id:        N       int         项目 ID
        :param id:                  O       int         id
        :param name:                O       str         用例名称
        :param description:         O       str         需求分类描述
        :param version:             O       str         版本
        :param owner:               O       str         测试负责人
        :param status:              O       str         用例状态 enum('close','open')
        :param type:                O       str         测试类型
        :param start_date:          O       date        开始时间
        :param end_date:            O       date        结束时间
        :param creator:             O       str         创建人
        :param created:             O       datetime    创建时间
        :param modified:            O       datetime    最后修改时间
        :param modifier:            O       string      最后修改人
        :return:
        """
        url = 'https://api.tapd.cn/test_plans/count?'
        param = tapd_param.TapdParam()
        param.setValue(workspace_id, int, 'workspace_id', True, {param.Int: {param.gt: 0}})
        param.setValue(id, int, 'id')
        param.setValue(name, str, 'name')
        param.setValue(description, str, 'description')
        param.setValue(version, str, 'version')
        param.setValue(owner, str, 'owner')
        param.setValue(status, str, 'status')
        param.setValue(type, str, 'type')
        param.setValue(creator, str, 'creator')
        param.setValue(modifier, str, 'modifier')
        self.wait()
        url = '%s%s' % (url, urlencode(param.data))
        url += self.dateCombine('start_date', start_date)
        url += self.dateCombine('end_date', end_date)
        url += self.dateTimeCombine('created', created)
        url += self.dateTimeCombine('modified', modified)
        # print(url)
        _json = HttpSimple(url).addMethod(HttpSimple.method_get).addAuth(self.retAuth()).run().retJson()
        self.checkApiStatus(_json)
        return _json.IntP('data/count')


if __name__ == '__main__':
    import datetime

    start_date = datetime.datetime.now().date() - datetime.timedelta(1)
    end_date = datetime.datetime.now().date() + datetime.timedelta(1)
    created = datetime.datetime.now() - datetime.timedelta(1)
    created_e = datetime.datetime.now() + datetime.timedelta(1)
    tapd_user = 'XXXX'
    tapd_pwd = 'XXXX'
    workspace_id = 123456
    t = TapdTestPlan(tapd_user, tapd_pwd).countTestPlan(workspace_id, status='open',
                                                        created={'btime': created, 'etime': created_e})
    print(t)
    pass
